#!/bin/bash
# -------
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along 
# with this program; if not, write to the Free Software Foundation, Inc., 
# 51 Franklin St, Fifth Floor, Boston MA 02110-1301 USA
# --------

set -e

#
# Added DATAFILE to store some values about ldap_tree
#

DATAFILE="/usr/share/lliurex/lliurex-userfuncs/ldap_tree.sh"
BLACKLIST_GROUPS="sabayon-admin nogroup"
#if [ -e $DATAFILE ]; then

#.$DATAFILE

#fi




llx_test_user() {
   [ -z "$1" ] && return 1
   getent passwd |egrep -q "^${1}\:" && return 0
   return 1
}

llx_test_group() {
   [ -z "$1" ] && return 1
   getent group |egrep -q "^${1}\:" && return 0
   return 1
}
llx_get_username() {
   # returns the username of the FIRST user with the required uid
   USER_ID="$1"
   if getent passwd |sed -ne "0,/^\([^:]*:\)\{2\}${USER_ID}/s%^\([^:]*\):[^:]*:${USER_ID}:.*$%\1%p"; then
      return 0
   fi
   echo ""
   return 1
}
llx_get_groupname(){
   # returns the groupname of the FIRST group with the required gid
   getent group |sed -ne "0,/^\([^:]*:\)\{2\}${1}/s%^\([^:]*\):[^:]*:${1}:.*$%\1%p"
}

llx_get_groupgid(){
   getent group |sed -ne "0,/^$1/s%^$1:[^:]*:\([^:]*\):.*$%\1%p"
}
llx_get_userid(){
   # returns the userid of the FIRST user with the required username
   getent passwd |sed -ne "0,/^$1/s%^$1:[^:]*:\([^:]*\):.*$%\1%p"
}
llx_get_usergid(){
   # returns the usergid of the FIRST user with the required username
   getent passwd |sed -ne "0,/^$1/s%^$1:\([^:]*:\)\{2\}\([^:]*\):.*$%\2%p"
}
llx_get_userhome(){
    getent passwd |sed -ne "0,/^$1/s%^$1:\([^:]*:\)\{4\}\([^:]*\):.*$%\2%p"
}
llx_get_user_maingroup(){
   llx_get_groupname `llx_get_usergid $1`
}

llx_get_user_groups() {
   # ojo, devuelve la lista completa SIN eliminar el caso de que exista un grupo con el mismo nombre que el usuario
   local LOCAL_USER="$1"
   llx_test_user "${LOCAL_USER}" &&  groups ${LOCAL_USER} |cut -f 2 -d :
   return 0
}

llx_test_user_in_group() {
   _llx_testlist "$2" `llx_get_user_groups $1` && return 0
   return 1
}

llx_get_default_group_list() {
   local GROUP_LIST USER_ID="`llx_get_username 1000`"
   # obtiene la lista de grupos, eliminando el que tiene el mismo nombre que el usuario
   GROUP_LIST="`llx_get_user_groups ${USER_ID} |sed -e s%${USER_ID}%%`"
   [ -z "$GROUP_LIST" ] && GROUP_LIST="dialout cdrom floppy audio video plugdev"

   # NO la separa con comas
   # echo  $GROUP_LIST |cut --delimiter=\  --output-delimiter=, -f 1-
   echo "$GROUP_LIST"
   return 0
}

llx_add_to_sudoers(){
   TARGET_USER="$1"
   SUDO_CONF="/etc/sudoers"
   TEMP_FILE=`tempfile`
   grep -v "^${TARGET_USER}[[:space:]]" ${SUDO_CONF} > "${TEMP_FILE}"
   if [ -z "$2" ] ; then
      # with password ...
      grep "^root" ${SUDO_CONF} |sed -e "s/root/${TARGET_USER}/" >> "${TEMP_FILE}"
   else
      # with NOPASSWD
       echo "${TARGET_USER}  ALL=NOPASSWD: ALL" >> "${TEMP_FILE}"
   fi
   cat ${TEMP_FILE} > ${SUDO_CONF}
   return 0
}

llx_add_users_to_sudoers(){
   local u USER_LIST NOPASSWD=""
   if [ "$1" = "NOPASSWD" ] ; then
      NOPASSWD="Y"
      shift
   fi
   USER_LIST="$@"
   for u in $USER_LIST; do
      llx_add_to_sudoers "${u}" ${NOPASSWD}
   done
}

llx_add_groups_to_sudoers(){
   local g GROUP_LIST NOPASSWD=""
   if [ "$1" = "NOPASSWD" ] ; then
      NOPASSWD="Y"
      shift
   fi
   GROUP_LIST="$@"
   for g in $GROUP_LIST; do
      llx_add_to_sudoers "%${g}" ${NOPASSWD}
   done
}
_canonicalize_cmd(){
   local NEW_CMD
   if [ "$1" ] ; then
      NEW_CMD="`readlink -f $1`"
      if [ -x "${NEW_CMD}" ] ; then
         shift
	 echo "${NEW_CMD} $@"
         return 0
      fi
   fi
   echo ""
}

llx_set_sudoers_cmd(){
   _llx_set_sudoers_cmd PASSWD "$@"
}

llx_set_sudoers_cmd_nopasswd(){
   _llx_set_sudoers_cmd NOPASSWD "$@"
}

_llx_set_sudoers_cmd(){
   TEMP_FILE=`tempfile`
   CMD_PREFIX="CMD_"
   LLXCMD_PREFIX="# LLiureX Cmnd alias specification"
   LLXGROUP_PREFIX="# LliureX groups specs"
   DEFAULT_PREFIX="# Cmnd alias specification"
   PASS_OPTION="$1"
   [ -z "${PASS_OPTION}" ] && return 1
   echo " PASSWD NOPASSWD " |grep -q " ${PASS_OPTION} " || return 1
   shift
   GROUP_NAME="$1"
   [ -z "${GROUP_NAME}" ] && return 1
   shift
   CMD_ALIAS="`echo "${CMD_PREFIX}${GROUP_NAME}" |tr "[:lower:]" "[:upper:]"`"
   CMD_LIST="`_canonicalize_cmd $1`"
   [ -z "${CMD_LIST}" ] && return 1
   shift
   while [ "$1" ]; do
      CMD_OK="`_canonicalize_cmd $1`"
      [ "${CMD_OK}" ] && CMD_LIST="${CMD_LIST}, ${1}"
      shift
   done
   # copy /etc/sudoers to TEMP_FILE adding LLXCMD_PREFIX line if needed
   if ! grep -q "^${LLXCMD_PREFIX}" /etc/sudoers ; then
       if ! grep -q "^${DEFAULT_PREFIX}" /etc/sudoers ; then
          echo "${LLXCMD_PREFIX}" >  ${TEMP_FILE}
	  cat /etc/sudoers >> ${TEMP_FILE}
       else
          sed -e "s%^${DEFAULT_PREFIX}%${DEFAULT_PREFIX}\n${LLXCMD_PREFIX}%" /etc/sudoers > ${TEMP_FILE}
       fi
   else
      cat /etc/sudoers > ${TEMP_FILE}
   fi
   # add LLXGROUP_PREFIX line to TEMP_FILE
   if ! grep -q "^${LLXGROUP_PREFIX}" ${TEMP_FILE} ; then
      echo "${LLXGROUP_PREFIX}" >>  ${TEMP_FILE}
   fi
   CMD_LINE="Cmnd_Alias ${CMD_ALIAS}=${CMD_LIST}"
   GROUP_LINE="%${GROUP_NAME} ALL=${PASS_OPTION}: ${CMD_ALIAS}"
   # remove old Cmnd_Alias entry
   SED_CMD="/^Cmnd_Alias[[:space:]]\+${CMD_ALIAS}=.*/d"
   # remove old group entry
   SED_CMD="${SED_CMD};/^%${GROUP_NAME}[[:space:]]\+ALL=\(NO\)\?PASSWD:[[:space:]]${CMD_ALIAS}/d"
   # add Cmnd_Alias entry
   SED_CMD="${SED_CMD};s%^${LLXCMD_PREFIX}%&\n${CMD_LINE}%"
   # add group entry
   SED_CMD="${SED_CMD};s/^${LLXGROUP_PREFIX}/&\n${GROUP_LINE}/"
   
   # exec SED_CMD and install the new /etc/sudoers file
   sed -e "${SED_CMD}" ${TEMP_FILE} | llxcfg-install - /etc/sudoers
   rm -f ${TEMP_FILE}

}

llx_set_userpasswd(){
   TARGET_USER="$1"
   NEW_PASS="$2"
   if llx_test_user "${TARGET_USER}" ; then 
      usermod -U ${TARGET_USER}
      usermod -p `mkpasswd $NEW_PASS` ${TARGET_USER}
   fi
   return 0
}

llx_del_userpasswd(){
   TARGET_USER="$1"
   # borrar passwords
   TMP_FILE="`tempfile`"
   SHADOW_FILE="/etc/shadow"
   sed  s/^${TARGET_USER}\:[^\:]*/${TARGET_USER}\:/ ${SHADOW_FILE} > ${TMP_FILE}
   cat ${TMP_FILE} > ${SHADOW_FILE}
   return 0
}

llx_test_local_user(){
   [ -z "$1" ] && return 1
   grep -q "^${1}:.*" /etc/passwd
}

llx_get_group_gid(){
   getent group $1 |cut -d : -f 3   
}

_llx_get_group_members(){
   local EXTRA_MEMBERS GROUP_GID GROUP_NAME="$1"
   llx_test_group "$GROUP_NAME" || return 0
   GROUP_GID=`llx_get_group_gid $GROUP_NAME`
   # obtiene usuarios con este grupo principal
   getent passwd |sed -ne "/^\([^:]*:\)\{3\}${GROUP_GID}:.*$/p" |cut -f 1 -d :
   # obtiene usuarios adicionales
   EXTRA_MEMBERS=`getent group $GROUP_NAME |cut -d : -f 4 |cut -f 1- -d , --output-delimiter "\n"`
   echo -e $EXTRA_MEMBERS
}

llx_get_group_members(){
   _llx_get_group_members $1 |sort -u
}

_llx_testlist(){
   # $1 item $2 list
   local SEEK_ITEM="$1"
   shift
   echo " $@ " |tr "\n" " " |grep -q " $SEEK_ITEM " && return 0
   return 1
}

_remove_blacklisted_groups(){
   local sed_cmd
   for p in `echo $BLACKLIST_GROUPS` ; do
      sed_cmd="/^$p/D;$sed_cmd"
   done

   echo "$sed_cmd"
   return 0
}

llx_get_non_system_groups(){
	
	getent group | awk 'BEGIN {FS=":"} $3>=1000 {print $1 }'|sed -e "$(_remove_blacklisted_groups)"

}
